@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21788 $
+ * $Revision: 23806 $
  * $Author: rjongbloed $
- * $Date: 2008-12-11 23:42:13 -0600 (Thu, 11 Dec 2008) $
+ * $Date: 2009-11-27 00:59:05 -0600 (Fri, 27 Nov 2009) $
  */
 
 #ifndef PTLIB_HTTP_H
@@ -41,6 +41,7 @@
 #include <ptclib/mime.h>
 #include <ptclib/url.h>
 #include <ptlib/ipsock.h>
+#include <ptlib/pfactory.h>
 
 
 #include <ptclib/html.h>
@@ -150,7 +151,7 @@ class PHTTPSpace : public PContainer
     PBoolean SetSize(PINDEX) { return PFalse; }
 };
 
-#ifdef _WIN32_WCE
+#ifdef TRACE
 #undef TRACE
 #endif
 
@@ -217,7 +218,7 @@ class PHTTP : public PInternetProtocol
     static const PString & AuthorizationTag();
     static const PString & ContentEncodingTag();
     static const PString & ContentLengthTag();
-    static const PString & ContentTypeTag();
+    static const PString & ContentTypeTag() { return PMIMEInfo::ContentTypeTag(); }
     static const PString & DateTag();
     static const PString & ExpiresTag();
     static const PString & FromTag();
@@ -264,6 +265,151 @@ class PHTTP : public PInternetProtocol
 };
 
 
+
+class PHTTPClientAuthentication : public PObject
+{
+  PCLASSINFO(PHTTPClientAuthentication, PObject);
+  public:
+    class AuthObject {
+      public:
+        virtual ~AuthObject() { }
+        virtual PMIMEInfo & GetMIME() = 0;
+        virtual PString GetURI() = 0;
+        virtual PString GetEntityBody() = 0;
+        virtual PString GetMethod() = 0;
+    };
+
+    PHTTPClientAuthentication();
+
+    virtual Comparison Compare(
+      const PObject & other
+    ) const;
+
+    virtual PBoolean Parse(
+      const PString & auth,
+      PBoolean proxy
+    ) = 0;
+
+    virtual PBoolean Authorise(
+      AuthObject & pdu
+    ) const =  0;
+
+    virtual PBoolean IsProxy() const               { return isProxy; }
+
+    virtual PString GetUsername() const   { return username; }
+    virtual PString GetPassword() const   { return password; }
+    virtual PString GetAuthRealm() const  { return PString::Empty(); }
+
+    virtual void SetUsername(const PString & user) { username = user; }
+    virtual void SetPassword(const PString & pass) { password = pass; }
+    virtual void SetAuthRealm(const PString &)     { }
+
+    PString GetAuthParam(const PString & auth, const char * name) const;
+    PString AsHex(PMessageDigest5::Code & digest) const;
+    PString AsHex(const PBYTEArray & data) const;
+
+    static PHTTPClientAuthentication * ParseAuthenticationRequired(bool isProxy, const PMIMEInfo & line, PString & errorMsg);
+
+
+  protected:
+    PBoolean  isProxy;
+    PString   username;
+    PString   password;
+};
+
+typedef PFactory<PHTTPClientAuthentication> PHTTPClientAuthenticationFactory;
+
+class PHTTPClientAuthenticator : public PHTTPClientAuthentication::AuthObject
+{
+  public:
+    PHTTPClientAuthenticator(
+      const PString & cmdName, 
+      const PString & uri, 
+      PMIMEInfo & mime, 
+      const PString & body
+    );
+    virtual PMIMEInfo & GetMIME();
+    virtual PString GetURI();
+    virtual PString GetEntityBody();
+    virtual PString GetMethod();
+  protected:
+    PString m_method;
+    PString m_uri;
+    PMIMEInfo & m_mime;
+    PString m_body;
+};
+
+///////////////////////////////////////////////////////////////////////
+
+class PHTTPClientBasicAuthentication : public PHTTPClientAuthentication
+{
+  PCLASSINFO(PHTTPClientBasicAuthentication, PHTTPClientAuthentication);
+  public:
+    PHTTPClientBasicAuthentication();
+
+    virtual Comparison Compare(
+      const PObject & other
+    ) const;
+
+    virtual PBoolean Parse(
+      const PString & auth,
+      PBoolean proxy
+    );
+
+    virtual PBoolean Authorise(
+      AuthObject & pdu
+    ) const;
+};
+
+///////////////////////////////////////////////////////////////////////
+
+class PHTTPClientDigestAuthentication : public PHTTPClientAuthentication
+{
+  PCLASSINFO(PHTTPClientDigestAuthentication, PHTTPClientAuthentication);
+  public:
+    PHTTPClientDigestAuthentication();
+
+    PHTTPClientDigestAuthentication & operator =(
+      const PHTTPClientDigestAuthentication & auth
+    );
+
+    virtual Comparison Compare(
+      const PObject & other
+    ) const;
+
+    virtual PBoolean Parse(
+      const PString & auth,
+      PBoolean proxy
+    );
+
+    virtual PBoolean Authorise(
+      AuthObject & pdu
+    ) const;
+
+    virtual PString GetAuthRealm() const         { return authRealm; }
+    virtual void SetAuthRealm(const PString & r) { authRealm = r; }
+
+    enum Algorithm {
+      Algorithm_MD5,
+      NumAlgorithms
+    };
+    const PString & GetNonce() const       { return nonce; }
+    Algorithm GetAlgorithm() const         { return algorithm; }
+    const PString & GetOpaque() const      { return opaque; }
+
+  protected:
+    PString   authRealm;
+    PString   nonce;
+    Algorithm algorithm;
+    PString   opaque;
+
+    PBoolean qopAuth;
+    PBoolean qopAuthInt;
+    PString cnonce;
+    mutable PAtomicInteger nonceCount;
+};
+
+
 //////////////////////////////////////////////////////////////////////////////
 // PHTTPClient
 
@@ -293,9 +439,8 @@ class PHTTPClient : public PHTTP
 
   public:
     /// Create a new HTTP client channel.
-    PHTTPClient();
     PHTTPClient(
-      const PString & userAgentName
+      const PString & userAgentName = PString::Empty()
     );
 
 
@@ -312,16 +457,14 @@ class PHTTPClient : public PHTTP
       const PURL & url,
       PMIMEInfo & outMIME,
       const PString & dataBody,
-      PMIMEInfo & replyMime,
-      PBoolean persist = PTrue
+      PMIMEInfo & replyMime
     );
     int ExecuteCommand(
       const PString & cmdName,
       const PURL & url,
       PMIMEInfo & outMIME,
       const PString & dataBody,
-      PMIMEInfo & replyMime,
-      PBoolean persist = PTrue
+      PMIMEInfo & replyMime
     );
 
     /// Write a HTTP command to server
@@ -356,13 +499,16 @@ class PHTTPClient : public PHTTP
 
     /** Get the document specified by the URL.
 
+        An empty string for the contentType parameter means that any content
+        type is acceptable.
+
        @return
        PTrue if document is being transferred.
      */
     PBoolean GetTextDocument(
       const PURL & url,         ///< Universal Resource Locator for document.
       PString & document,       ///< Body read
-      PBoolean persist = PTrue       ///< if PTrue, enable HTTP persistence
+      const PString & contentType = PString::Empty() ///< Content-Type header to expect
     );
 
     /** Get the document specified by the URL.
@@ -373,8 +519,7 @@ class PHTTPClient : public PHTTP
     PBoolean GetDocument(
       const PURL & url,         ///< Universal Resource Locator for document.
       PMIMEInfo & outMIME,      ///< MIME info in request
-      PMIMEInfo & replyMIME,    ///< MIME info in response
-      PBoolean persist = PTrue       ///< if PTrue, enable HTTP persistence
+      PMIMEInfo & replyMIME     ///< MIME info in response
     );
 
     /** Get the header for the document specified by the URL.
@@ -385,8 +530,7 @@ class PHTTPClient : public PHTTP
     PBoolean GetHeader(
       const PURL & url,         ///< Universal Resource Locator for document.
       PMIMEInfo & outMIME,      ///< MIME info in request
-      PMIMEInfo & replyMIME,    ///< MIME info in response
-      PBoolean persist = PTrue       ///< if PTrue, enable HTTP persistence
+      PMIMEInfo & replyMIME     ///< MIME info in response
     );
 
 
@@ -399,8 +543,7 @@ class PHTTPClient : public PHTTP
       const PURL & url,       ///< Universal Resource Locator for document.
       PMIMEInfo & outMIME,    ///< MIME info in request
       const PString & data,   ///< Information posted to the HTTP server.
-      PMIMEInfo & replyMIME,  ///< MIME info in response
-      PBoolean persist = PTrue     ///< if PTrue, enable HTTP persistence
+      PMIMEInfo & replyMIME   ///< MIME info in response
     );
 
     /** Post the data specified to the URL.
@@ -413,36 +556,69 @@ class PHTTPClient : public PHTTP
       PMIMEInfo & outMIME,    ///< MIME info in request
       const PString & data,   ///< Information posted to the HTTP server.
       PMIMEInfo & replyMIME,  ///< MIME info in response
-      PString & replyBody,    ///< Body of response
-      PBoolean persist = PTrue     ///< if PTrue, enable HTTP persistence
+      PString & replyBody     ///< Body of response
     );
 
+    /** Put the document specified by the URL.
+
+       @return
+       true if document is being transferred.
+     */
+    bool PutTextDocument(
+      const PURL & url,         ///< Universal Resource Locator for document.
+      const PString & document,       ///< Body read
+      const PString & contentType = "text/plain" ///< Content-Type header to use
+    );
+
+    /** Put the document specified by the URL.
+
+       @return
+       true if document is being transferred.
+     */
+    bool PutDocument(
+      const PURL & url,         ///< Universal Resource Locator for document.
+      PMIMEInfo & outMIME,      ///< MIME info in request
+      PMIMEInfo & replyMIME     ///< MIME info in response
+    );
+
+    /** Delete the document specified by the URL.
+
+       @return
+       true if document is deleted.
+     */
+    bool DeleteDocument(
+      const PURL & url        ///< Universal Resource Locator for document.
+    );
+
+    /** Set authentication paramaters to be use for retreiving documents
+    */
+    void SetAuthenticationInfo(
+      const PString & userName,
+      const PString & password
+    );
+
+    /// Set persistent connection mode
+    void SetPersistent(
+      bool persist = true
+    ) { m_persist = persist; }
+
+    /// Get persistent connection mode
+    bool GetPersistent() const { return m_persist; }
+
   protected:
     PBoolean AssureConnect(const PURL & url, PMIMEInfo & outMIME);
-    PBoolean InternalReadContentBody(
+    bool InternalReadContentBody(
       PMIMEInfo & replyMIME,
-      PAbstractArray & body
+      PAbstractArray * body
     );
 
-    PString userAgentName;
+    PString m_userAgentName;
+    bool    m_persist;
+    PString m_userName;
+    PString m_password;
+    PHTTPClientAuthentication * m_authentication;
 };
 
-//////////////////////////////////////////////////////////////////////////////
-// PMultipartFormInfo
-
-/** This object describes the information associated with a multi-part
-    form entry
-  */
-
-class PMultipartFormInfo : public PObject
-{
-  PCLASSINFO(PMultipartFormInfo, PObject);
-  public:
-    PMIMEInfo mime;
-    PString body;
-};
-
-PARRAY(PMultipartFormInfoArray, PMultipartFormInfo);
 
 //////////////////////////////////////////////////////////////////////////////
 // PHTTPConnectionInfo
@@ -495,17 +671,17 @@ class PHTTPConnectionInfo : public PObject
       */
     void SetPersistenceMaximumTransations(unsigned m) { persistenceMaximum = m; }
 
-    const PMultipartFormInfoArray & GetMultipartFormInfo() const
-      { return multipartFormInfoArray; }
+    const PMultiPartList & GetMultipartFormInfo() const
+      { return m_multipartFormInfo; }
 
     void ResetMultipartFormInfo()
-      { multipartFormInfoArray.RemoveAll(); }
+      { m_multipartFormInfo.RemoveAll(); }
 
     PString GetEntityBody() const   { return entityBody; }
 
   protected:
     PBoolean Initialise(PHTTPServer & server, PString & args);
-    void DecodeMultipartFormInfo(const PString & type, const PString & entityBody);
+    bool DecodeMultipartFormInfo() { return mimeInfo.DecodeMultiPartList(m_multipartFormInfo, entityBody); }
 
     PHTTP::Commands commandCode;
     PString         commandName;
@@ -520,7 +696,7 @@ class PHTTPConnectionInfo : public PObject
     long            entityBodyLength;
     PTimeInterval   persistenceTimeout;
     unsigned        persistenceMaximum;
-    PMultipartFormInfoArray multipartFormInfoArray;
+    PMultiPartList  m_multipartFormInfo;
 
   friend class PHTTPServer;
 };
@@ -755,14 +931,15 @@ class PHTTPRequest : public PObject
     PHTTPRequest(
       const PURL & url,             ///< Universal Resource Locator for document.
       const PMIMEInfo & inMIME,     ///< Extra MIME information in command.
-      const PMultipartFormInfoArray & multipartFormInfo, ///< multipart form information (if any)
+      const PMultiPartList & multipartFormInfo, ///< multipart form information (if any)
+      PHTTPResource * resource,     ///< Resource associated with request
       PHTTPServer & server          ///< Server channel that request initiated on
     );
 
     PHTTPServer & server;           ///< Server channel that request initiated on
     const PURL & url;               ///< Universal Resource Locator for document.
     const PMIMEInfo & inMIME;       ///< Extra MIME information in command.
-    const PMultipartFormInfoArray & multipartFormInfo; ///< multipart form information, if any
+    const PMultiPartList & multipartFormInfo; ///< multipart form information, if any
     PHTTP::StatusCode code;         ///< Status code for OnError() reply.
     PMIMEInfo outMIME;              ///< MIME information used in reply.
     PString entityBody;             ///< original entity body (POST only)
@@ -770,6 +947,7 @@ class PHTTPRequest : public PObject
     PIPSocket::Address origin;      ///< IP address of origin host for request
     PIPSocket::Address localAddr;   ///< IP address of local interface for request
     WORD               localPort;   ///< Port number of local server for request
+    PHTTPResource    * m_resource;  ///< HTTP resource found for the request
 };
 
 
@@ -1187,7 +1365,7 @@ class PHTTPResource : public PObject
     virtual PHTTPRequest * CreateRequest(
       const PURL & url,                   ///< Universal Resource Locator for document.
       const PMIMEInfo & inMIME,           ///< Extra MIME information in command.
-      const PMultipartFormInfoArray & multipartFormInfo,  ///< additional information for multi-part posts
+      const PMultiPartList & multipartFormInfo,  ///< additional information for multi-part posts
       PHTTPServer & socket                                ///< socket used for request
     );
 
@@ -1443,7 +1621,7 @@ class PHTTPFile : public PHTTPResource
     virtual PHTTPRequest * CreateRequest(
       const PURL & url,                  // Universal Resource Locator for document.
       const PMIMEInfo & inMIME,          // Extra MIME information in command.
-      const PMultipartFormInfoArray & multipartFormInfo,
+      const PMultiPartList & multipartFormInfo,
       PHTTPServer & socket
     );
 
@@ -1500,7 +1678,8 @@ class PHTTPFileRequest : public PHTTPRequest
     PHTTPFileRequest(
       const PURL & url,             // Universal Resource Locator for document.
       const PMIMEInfo & inMIME,     // Extra MIME information in command.
-      const PMultipartFormInfoArray & multipartFormInfo,
+      const PMultiPartList & multipartFormInfo,
+      PHTTPResource * resource,
       PHTTPServer & server
     );
 
@@ -1624,7 +1803,7 @@ class PHTTPDirectory : public PHTTPFile
     virtual PHTTPRequest * CreateRequest(
       const PURL & url,                  // Universal Resource Locator for document.
       const PMIMEInfo & inMIME,          // Extra MIME information in command.
-      const PMultipartFormInfoArray & multipartFormInfo,
+      const PMultiPartList & multipartFormInfo,
       PHTTPServer & socket
     );
 
@@ -1687,7 +1866,8 @@ class PHTTPDirRequest : public PHTTPFileRequest
     PHTTPDirRequest(
       const PURL & url,             // Universal Resource Locator for document.
       const PMIMEInfo & inMIME,     // Extra MIME information in command.
-      const PMultipartFormInfoArray & multipartFormInfo, 
+      const PMultiPartList & multipartFormInfo, 
+      PHTTPResource * resource,
       PHTTPServer & server
     );
 
