@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21788 $
+ * $Revision: 23931 $
  * $Author: rjongbloed $
- * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
+ * $Date: 2010-01-13 06:19:08 -0600 (Wed, 13 Jan 2010) $
  */
 
 #ifndef PTLIB_IPSOCKET_H
@@ -48,6 +48,11 @@
 #endif // _WIN32
 #endif // P_QOS
 
+
+class PIPSocketAddressAndPort;
+
+
+
 /**This class describes a type of socket that will communicate using the
    Internet Protocol.
    If P_HAS_IPV6 is not set, IPv4 only is supported.
@@ -218,6 +223,9 @@ class PIPSocket : public PSocket
         /// Check for Broadcast address 255.255.255.255.
         PBoolean IsBroadcast() const;
 
+        /// Check if address is multicast group
+        PBoolean IsMulticast() const;
+
         /** Check if the remote address is a private address.
             For IPV4 this is specified RFC 1918 as the following ranges:
             \li    10.0.0.0 - 10.255.255.255.255
@@ -428,6 +436,9 @@ class PIPSocket : public PSocket
     virtual PBoolean GetLocalAddress(
       Address & addr    ///< Variable to receive hosts IP address.
     );
+    virtual PBoolean GetLocalAddress(
+      PIPSocketAddressAndPort & addr    ///< Variable to receive hosts IP address and port.
+    );
     virtual PBoolean GetLocalAddress(
       Address & addr,    ///< Variable to receive peer hosts IP address.
       WORD & port        ///< Variable to receive peer hosts port number.
@@ -443,6 +454,9 @@ class PIPSocket : public PSocket
     virtual PBoolean GetPeerAddress(
       Address & addr    ///< Variable to receive hosts IP address.
     );
+    virtual PBoolean GetPeerAddress(
+      PIPSocketAddressAndPort & addr    ///< Variable to receive hosts IP address and port.
+    );
     virtual PBoolean GetPeerAddress(
       Address & addr,    ///< Variable to receive peer hosts IP address.
       WORD & port        ///< Variable to receive peer hosts port number.
@@ -583,14 +597,26 @@ class PIPSocket : public PSocket
       RouteTable & table      ///< Route table
 	);
 
-    /** Wait for a change in the route table or network interfaces.
+    /// Class for detector of Route Table changes
+    class RouteTableDetector
+    {
+      public:
+        virtual ~RouteTableDetector() { }
+        virtual bool Wait(
+          const PTimeInterval & timeout ///< Time to wait for refresh (may be ignored)
+        ) = 0;
+        virtual void Cancel() = 0;
+    };
 
-        @return true if a change occurred, false if timed out or cancelled.
+    /** Create an object that can wait for a change in the route table or
+        active network interfaces.
+
+        If the platform does not support this mechanism then a fake class is
+        created using PSyncPoint to wait for the specified amount of time.
+
+        @return Pointer to some object, never returns NULL.
       */
-    static bool WaitForRouteTableChange(
-      const PTimeInterval & timeout,    ///< Timeout for wait
-      PSyncPoint * cancellation = NULL  ///< if sync point is signalled, wait is immediately terminated
-    );
+    static RouteTableDetector * CreateRouteTableDetector();
 
     /**Describe an interface table entry.
      */
@@ -729,8 +755,15 @@ class PIPSocketAddressAndPort
     PString AsString(char separator = 0) const
       { return m_address.AsString() + (separator ? separator : m_separator) + PString(PString::Unsigned, m_port); }
 
+    void SetAddress(
+      const PIPSocket::Address & addr,
+      WORD port = 0
+    );
     const PIPSocket::Address & GetAddress() const { return m_address; }
     WORD GetPort() const { return m_port; }
+    void SetPort(
+      WORD port
+    ) { m_port = port; }
 
   protected:
     PIPSocket::Address m_address;
