@@ -26,13 +26,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20492 $
+ * $Revision: 23897 $
  * $Author: rjongbloed $
- * $Date: 2008-06-25 01:36:32 +0200 (mer 25 jun 2008) $
+ * $Date: 2009-12-23 19:02:12 -0600 (Wed, 23 Dec 2009) $
  */
 
-#ifndef _POBJECT_H
-#define _POBJECT_H
+#ifndef PTLIB_OBJECT_H
+#define PTLIB_OBJECT_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -60,6 +60,36 @@
 #include <sstream>
 
 
+#define P_REMOVE_VIRTUAL_INTERNAL_BASE(fn) __inline virtual struct ptlib_virtual_function_changed_or_removed ****** fn { return 0; }
+
+#if defined(_MSC_VER)
+  #if _MSC_VER < 1310
+    #define P_DEPRECATED
+    #define P_REMOVE_VIRTUAL_INTERNAL(type, fn, body) P_REMOVE_VIRTUAL_INTERNAL_BASE(fn)
+  #elif _MSC_VER < 1400
+    #define P_DEPRECATED __declspec(deprecated)
+    #define P_REMOVE_VIRTUAL_INTERNAL(type, fn, body) __inline virtual __declspec(deprecated) type fn body
+  #else
+    #define P_DEPRECATED __declspec(deprecated)
+    #define P_REMOVE_VIRTUAL_INTERNAL(type, fn, body) __inline virtual __declspec(deprecated("Virtual function signature changed or function deprecated")) type fn body
+  #endif
+#elif defined(__GNUC__)
+  #if __GNUC__ < 4
+    #define P_DEPRECATED
+    #define P_REMOVE_VIRTUAL_INTERNAL(type, fn, body) P_REMOVE_VIRTUAL_INTERNAL_BASE(fn)
+  #else
+    #define P_DEPRECATED __attribute__((deprecated))
+    #define P_REMOVE_VIRTUAL_INTERNAL(type, fn, body) __attribute__((warn_unused_result)) __attribute__((deprecated)) P_REMOVE_VIRTUAL_INTERNAL_BASE(fn)
+  #endif
+#else
+    #define P_DEPRECATED
+    #define P_REMOVE_VIRTUAL_INTERNAL(type, fn, body) P_REMOVE_VIRTUAL_INTERNAL_BASE(fn)
+#endif
+
+#define P_REMOVE_VIRTUAL_VOID(fn)       P_REMOVE_VIRTUAL_INTERNAL(void, fn, {})
+#define P_REMOVE_VIRTUAL(type, fn, ret) P_REMOVE_VIRTUAL_INTERNAL(type, fn, { return ret; })
+
+
 // P_USE_INTEGER_BOOL is the default and gives the old behaviour (it
 // is also used for C translation units).
 // without P_USE_INTEGER_BOOL, the ANSI C++ bool is used.
@@ -107,8 +137,8 @@
 #define PAssert2(b, c, m) (b)
 #define PAssertOS(b) (b)
 #define PAssertNULL(p) (p)
-#define PAssertAlways(m)
-#define PAssertAlways2(c, m)
+#define PAssertAlways(m) {}
+#define PAssertAlways2(c, m) {}
 
 #else // P_USE_ASSERTS
 
@@ -218,7 +248,7 @@ void PSetErrorStream(ostream * strm /** New stream for error output */ );
 
 /** This macro is used to access the platform specific error output stream.
 This is to be used in preference to assuming #cerr is always available. On
-Unix platforms this {\bfis} #cerr but for MS-Windows this is another stream
+Unix platforms this {\b is} #cerr but for MS-Windows this is another stream
 that uses the OutputDebugString() Windows API function. Note that a MS-DOS or
 Windows NT console application would still use #cerr.
 
@@ -275,15 +305,19 @@ public:
     /** Output timestamps in GMT time rather than local time
       */
     GMTTime = 256,
-    /** If set, log file will be rotated daily
+    /** If set, log file will be rotated 
       */
     RotateDaily = 512,
+    RotateHourly = 1024,
+    RotateMinutely = 2048,
+    RotateLogMask = RotateDaily + RotateHourly + RotateMinutely,
     /** SystemLog flag for tracing within a PServiceProcess application. Must
         be set in conjection with SetStream(new PSystemLog).
       */
     SystemLogStream = 32768
   };
 
+
   /**Set the most common trace options.
      If filename is not NULL then a PTextFile is created and attached the
      trace output stream. This object is never closed or deleted until the
@@ -912,6 +946,32 @@ public:
 #endif // PMEMORY_CHECK || (defined(_MSC_VER) && defined(_DEBUG))
 
 
+// Memory pooling allocators
+#if defined(__GNUC__) && __GNUC__ >= 4 && !defined(P_MINGW)
+#include <ext/bitmap_allocator.h>
+template <class Type> class PFixedPoolAllocator    : public __gnu_cxx::bitmap_allocator<Type> { };
+#include <ext/mt_allocator.h>
+template <class Type> class PVariablePoolAllocator : public __gnu_cxx::__mt_alloc<Type>   { };
+#else
+template <class Type> class PFixedPoolAllocator    : public std::allocator<Type> { };
+template <class Type> class PVariablePoolAllocator : public std::allocator<Type> { };
+#endif
+
+#define PDECLARE_POOL_ALLOCATOR() \
+    void * operator new(size_t nSize); \
+    void * operator new(size_t nSize, const char * file, int line); \
+    void operator delete(void * ptr); \
+    void operator delete(void * ptr, const char *, int)
+
+#define PDEFINE_POOL_ALLOCATOR(cls) \
+  static PFixedPoolAllocator<cls> cls##_allocator; \
+  void * cls::operator new(size_t)                           { return cls##_allocator.allocate(1);               } \
+  void * cls::operator new(size_t, const char *, int)        { return cls##_allocator.allocate(1);               } \
+  void   cls::operator delete(void * ptr)                    {        cls##_allocator.deallocate((cls *)ptr, 1); } \
+  void   cls::operator delete(void * ptr, const char *, int) {        cls##_allocator.deallocate((cls *)ptr, 1); }
+
+
+
 /** Declare all the standard PWlib class information.
 This macro is used to provide the basic run-time typing capability needed
 by the library. All descendent classes from the #PObject class require
@@ -1009,7 +1069,7 @@ class PObject {
     /** Get the name of the class as a C string. This is a static function which
        returns the type of a specific class. 
        
-       When comparing class names, always use the #strcmp()#
+       When comparing class names, always use the #strcmp()
        function rather than comparing pointers. The pointers are not
        necessarily the same over compilation units depending on the compiler,
        platform etc.
@@ -1020,7 +1080,7 @@ class PObject {
 
     /** Get the current dynamic type of the object instance.
 
-       When comparing class names, always use the #strcmp()#
+       When comparing class names, always use the #strcmp()
        function rather than comparing pointers. The pointers are not
        necessarily the same over compilation units depending on the compiler,
        platform etc.
@@ -1030,7 +1090,7 @@ class PObject {
 
        @return pointer to C string literal.
      */
-    virtual const char * GetClass(unsigned /*ancestor*/ = 0) const { return Class(); }
+    virtual const char * GetClass(unsigned ancestor = 0) const { return ancestor > 0 ? "" : Class(); }
 
     PBoolean IsClass(const char * cls) const 
     { return strcmp(cls, GetClass()) == 0; }
@@ -1053,7 +1113,7 @@ class PObject {
 
   /**@name Comparison functions */
   //@{
-    /** Result of the comparison operation performed by the #Compare()#
+    /** Result of the comparison operation performed by the #Compare()
        function.
       */
     enum Comparison {
@@ -1079,14 +1139,14 @@ class PObject {
     
     /** Determine the byte wise comparison of two objects. This is the default
        comparison operation for objects that do not explicitly override the
-       #Compare()# function.
+       #Compare() function.
     
        The #PCLASSINFO macro declares an override of this function for
        the particular class. The user need not implement it.
 
        @return
        #LessThan, #EqualTo or #GreaterThan
-       according to the result #memcpy()# function.
+       according to the result #memcpy() function.
      */
     virtual Comparison CompareObjectMemoryDirect(
       const PObject & obj   // Object to compare against.
@@ -1151,7 +1211,7 @@ class PObject {
   //@{
     /** Output the contents of the object to the stream. The exact output is
        dependent on the exact semantics of the descendent class. This is
-       primarily used by the standard #operator<<# function.
+       primarily used by the standard #operator<< function.
 
        The default behaviour is to print the class name.
      */
@@ -1161,7 +1221,7 @@ class PObject {
 
     /** Input the contents of the object from the stream. The exact input is
        dependent on the exact semantics of the descendent class. This is
-       primarily used by the standard #operator>># function.
+       primarily used by the standard #operator>> function.
 
        The default behaviour is to do nothing.
      */
@@ -1171,7 +1231,7 @@ class PObject {
 
 
     /** Global function for using the standard << operator on objects descended
-       from PObject. This simply calls the objects #PrintOn()# function.
+       from PObject. This simply calls the objects #PrintOn() function.
        
        @return the #strm parameter.
      */
@@ -1181,7 +1241,7 @@ class PObject {
     ) { obj.PrintOn(strm); return strm; }
 
     /** Global function for using the standard >> operator on objects descended
-       from PObject. This simply calls the objects #ReadFrom()# function.
+       from PObject. This simply calls the objects #ReadFrom() function.
 
        @return the #strm parameter.
      */
@@ -1195,7 +1255,7 @@ class PObject {
   //@{
     /** Create a copy of the class on the heap. The exact semantics of the
        descendent class determine what is required to make a duplicate of the
-       instance. Not all classes can even {\bf do} a clone operation.
+       instance. Not all classes can even \b do a clone operation.
        
        The main user of the clone function is the #PDictionary class as
        it requires copies of the dictionary keys.
@@ -1235,209 +1295,166 @@ class PObject {
 #define PBIG_ENDIAN 3
 
 
-#if 0
-class PStandardType
-/* Encapsulate a standard 8 bit character into a portable format. This would
-   rarely need to do translation, only if the target platform uses EBCDIC
-   would it do anything.
-
-   The platform independent form here is always 8 bit ANSI.
- */
-{
-  public:
-    PStandardType(
-      type newVal   // Value to initialise data in platform dependent form.
-    ) { data = newVal; }
-    /* Create a new instance of the platform independent type using platform
-       dependent data, or platform independent streams.
-     */
-
-    operator type() { return data; }
-    /* Get the platform dependent value for the type.
-
-       @return
-       data for instance.
-     */
-
-    friend ostream & operator<<(ostream & strm, const PStandardType & val)
-      { return strm << (type)val; }
-    /* Output the platform dependent value for the type to the stream.
-
-       @return
-       the stream output was made to.
-     */
-
-    friend istream & operator>>(istream & strm, PStandardType & val)
-      { type data; strm >> data; val = PStandardType(data); return strm; }
-    /* Input the platform dependent value for the type from the stream.
-
-       @return
-       the stream input was made from.
-     */
-
+template <typename type>
+struct PIntSameOrder {
+  __inline PIntSameOrder()                            : data(0)              { }
+  __inline PIntSameOrder(type value)                  : data(value)          { }
+  __inline PIntSameOrder(const PIntSameOrder & value) : data(value.data)     { }
+  __inline PIntSameOrder & operator=(type value)                             { data = value; return *this; }
+  __inline PIntSameOrder & operator=(const PIntSameOrder & value)            { data = value.data; return *this; }
+  __inline operator type() const                                             { return data; }
+  __inline friend ostream & operator<<(ostream & s, const PIntSameOrder & v) { return s << v.data; }
+  __inline friend istream & operator>>(istream & s, PIntSameOrder & v)       { return s >> v.data; }
 
   private:
     type data;
 };
-#endif
 
 
-#define PI_SAME(name, type) \
-  struct name { \
-    name() : data(0) { } \
-    name(type value) : data(value) { } \
-    name(const name & value) : data(value.data) { } \
-    name & operator =(type value) { data = value; return *this; } \
-    name & operator =(const name & value) { data = value.data; return *this; } \
-    operator type() const { return data; } \
-    friend ostream & operator<<(ostream & s, const name & v) { return s << v.data; } \
-    friend istream & operator>>(istream & s, name & v) { return s >> v.data; } \
-    private: type data; \
-  }
+template <typename type>
+struct PIntReversedOrder {
+  __inline PIntReversedOrder()                                : data(0)              { }
+  __inline PIntReversedOrder(type value)                                             { ReverseBytes(value, data); }
+  __inline PIntReversedOrder(const PIntReversedOrder & value) : data(value.data)     { }
+  __inline PIntReversedOrder & operator=(type value)                                 { ReverseBytes(value, data); return *this; }
+  __inline PIntReversedOrder & operator=(const PIntReversedOrder & value)            { data = value.data; return *this; }
+  __inline operator type() const                                                     { type value; ReverseBytes(data, value); return value; }
+  __inline friend ostream & operator<<(ostream & s, const PIntReversedOrder & value) { return s << (type)value; }
+  __inline friend istream & operator>>(istream & s, PIntReversedOrder & v)           { type val; s >> val; v = val; return s; }
 
-#define PI_LOOP(src, dst) \
-    BYTE *s = ((BYTE *)&src)+sizeof(src); BYTE *d = (BYTE *)&dst; \
-    while (s != (BYTE *)&src) *d++ = *--s;
+  private:
+    type data;
 
-#define PI_DIFF(name, type) \
-  struct name { \
-    name() : data(0) { } \
-    name(type value) : data(0) { operator=(value); } \
-    name(const name & value) : data(value.data) { } \
-    name & operator =(type value) { PI_LOOP(value, data); return *this; } \
-    name & operator =(const name & value) { data = value.data; return *this; } \
-    operator type() const { type value; PI_LOOP(data, value); return value; } \
-    friend ostream & operator<<(ostream & s, const name & value) { return s << (type)value; } \
-    friend istream & operator>>(istream & s, name & v) { type val; s >> val; v = val; return s; } \
-    private: type data; \
+  static __inline void ReverseBytes(const type & src, type & dst)
+  {
+    size_t s = sizeof(type)-1;
+    for (size_t d = 0; d < sizeof(type); ++d,--s)
+      ((BYTE *)&dst)[d] = ((const BYTE *)&src)[s];
   }
+};
 
 #ifndef PCHAR8
 #define PCHAR8 PANSI_CHAR
 #endif
 
 #if PCHAR8==PANSI_CHAR
-PI_SAME(PChar8, char);
+typedef PIntSameOrder<char> PChar8;
 #endif
 
-PI_SAME(PInt8, char);
+typedef PIntSameOrder<char> PInt8;
 
-PI_SAME(PUInt8, unsigned char);
+typedef PIntSameOrder<unsigned char> PUInt8;
 
 #if PBYTE_ORDER==PLITTLE_ENDIAN
-PI_SAME(PInt16l, PInt16);
+typedef PIntSameOrder<PInt16> PInt16l;
 #elif PBYTE_ORDER==PBIG_ENDIAN
-PI_DIFF(PInt16l, PInt16);
+typedef PIntReversedOrder<PInt16> PInt16l;
 #endif
 
 #if PBYTE_ORDER==PLITTLE_ENDIAN
-PI_DIFF(PInt16b, PInt16);
+typedef PIntReversedOrder<PInt16> PInt16b;
 #elif PBYTE_ORDER==PBIG_ENDIAN
-PI_SAME(PInt16b, PInt16);
+typedef PIntSameOrder<PInt16> PInt16b;
 #endif
 
 #if PBYTE_ORDER==PLITTLE_ENDIAN
-PI_SAME(PUInt16l, WORD);
+typedef PIntSameOrder<WORD> PUInt16l;
 #elif PBYTE_ORDER==PBIG_ENDIAN
-PI_DIFF(PUInt16l, WORD);
+typedef PIntReversedOrder<WORD> PUInt16l;
 #endif
 
 #if PBYTE_ORDER==PLITTLE_ENDIAN
-PI_DIFF(PUInt16b, WORD);
+typedef PIntReversedOrder<WORD> PUInt16b;
 #elif PBYTE_ORDER==PBIG_ENDIAN
-PI_SAME(PUInt16b, WORD);
+typedef PIntSameOrder<WORD> PUInt16b;
 #endif
 
 #if PBYTE_ORDER==PLITTLE_ENDIAN
-PI_SAME(PInt32l, PInt32);
+typedef PIntSameOrder<PInt32> PInt32l;
 #elif PBYTE_ORDER==PBIG_ENDIAN
-PI_DIFF(PInt32l, PInt32);
+typedef PIntReversedOrder<PInt32> PInt32l;
 #endif
 
 #if PBYTE_ORDER==PLITTLE_ENDIAN
-PI_DIFF(PInt32b, PInt32);
+typedef PIntReversedOrder<PInt32> PInt32b;
 #elif PBYTE_ORDER==PBIG_ENDIAN
-PI_SAME(PInt32b, PInt32);
+typedef PIntSameOrder<PInt32> PInt32b;
 #endif
 
 #if PBYTE_ORDER==PLITTLE_ENDIAN
-PI_SAME(PUInt32l, DWORD);
+typedef PIntSameOrder<DWORD> PUInt32l;
 #elif PBYTE_ORDER==PBIG_ENDIAN
-PI_DIFF(PUInt32l, DWORD);
+typedef PIntReversedOrder<DWORD> PUInt32l;
 #endif
 
 #if PBYTE_ORDER==PLITTLE_ENDIAN
-PI_DIFF(PUInt32b, DWORD);
+typedef PIntReversedOrder<DWORD> PUInt32b;
 #elif PBYTE_ORDER==PBIG_ENDIAN
-PI_SAME(PUInt32b, DWORD);
+typedef PIntSameOrder<DWORD> PUInt32b;
 #endif
 
 #if PBYTE_ORDER==PLITTLE_ENDIAN
-PI_SAME(PInt64l, PInt64);
+typedef PIntSameOrder<PInt64> PInt64l;
 #elif PBYTE_ORDER==PBIG_ENDIAN
-PI_DIFF(PInt64l, PInt64);
+typedef PIntReversedOrder<PInt64> PInt64l;
 #endif
 
 #if PBYTE_ORDER==PLITTLE_ENDIAN
-PI_DIFF(PInt64b, PInt64);
+typedef PIntReversedOrder<PInt64> PInt64b;
 #elif PBYTE_ORDER==PBIG_ENDIAN
-PI_SAME(PInt64b, PInt64);
+typedef PIntSameOrder<PInt64> PInt64b;
 #endif
 
 #if PBYTE_ORDER==PLITTLE_ENDIAN
-PI_SAME(PUInt64l, PUInt64);
+typedef PIntSameOrder<PUInt64> PUInt64l;
 #elif PBYTE_ORDER==PBIG_ENDIAN
-PI_DIFF(PUInt64l, PUInt64);
+typedef PIntReversedOrder<PUInt64> PUInt64l;
 #endif
 
 #if PBYTE_ORDER==PLITTLE_ENDIAN
-PI_DIFF(PUInt64b, PUInt64);
+typedef PIntReversedOrder<PUInt64> PUInt64b;
 #elif PBYTE_ORDER==PBIG_ENDIAN
-PI_SAME(PUInt64b, PUInt64);
+typedef PIntSameOrder<PUInt64> PUInt64b;
 #endif
 
 #if PBYTE_ORDER==PLITTLE_ENDIAN
-PI_SAME(PFloat32l, float);
+typedef PIntSameOrder<float> PFloat32l;
 #elif PBYTE_ORDER==PBIG_ENDIAN
-PI_DIFF(PFloat32l, float);
+typedef PIntReversedOrder<float> PFloat32l;
 #endif
 
 #if PBYTE_ORDER==PLITTLE_ENDIAN
-PI_DIFF(PFloat32b, float);
+typedef PIntReversedOrder<float> PFloat32b;
 #elif PBYTE_ORDER==PBIG_ENDIAN
-PI_SAME(PFloat32b, float);
+typedef PIntSameOrder<float> PFloat32b;
 #endif
 
 #if PBYTE_ORDER==PLITTLE_ENDIAN
-PI_SAME(PFloat64l, double);
+typedef PIntSameOrder<double> PFloat64l;
 #elif PBYTE_ORDER==PBIG_ENDIAN
-PI_DIFF(PFloat64l, double);
+typedef PIntReversedOrder<double> PFloat64l;
 #endif
 
 #if PBYTE_ORDER==PLITTLE_ENDIAN
-PI_DIFF(PFloat64b, double);
+typedef PIntReversedOrder<double> PFloat64b;
 #elif PBYTE_ORDER==PBIG_ENDIAN
-PI_SAME(PFloat64b, double);
+typedef PIntSameOrder<double> PFloat64b;
 #endif
 
 #ifndef NO_LONG_DOUBLE // stupid OSX compiler
 #if PBYTE_ORDER==PLITTLE_ENDIAN
-PI_SAME(PFloat80l, long double);
+typedef PIntSameOrder<long double> PFloat80l;
 #elif PBYTE_ORDER==PBIG_ENDIAN
-PI_DIFF(PFloat80l, long double);
+typedef PIntReversedOrder<long double> PFloat80l;
 #endif
 
 #if PBYTE_ORDER==PLITTLE_ENDIAN
-PI_DIFF(PFloat80b, long double);
+typedef PIntReversedOrder<long double> PFloat80b;
 #elif PBYTE_ORDER==PBIG_ENDIAN
-PI_SAME(PFloat80b, long double);
+typedef PIntSameOrder<long double> PFloat80b;
 #endif
 #endif
 
-#undef PI_LOOP
-#undef PI_SAME
-#undef PI_DIFF
-
 
 ///////////////////////////////////////////////////////////////////////////////
 // Miscellaneous
@@ -1469,6 +1486,8 @@ PI_SAME(PFloat80b, long double);
  */
 #define PABS(v) ((v) < 0 ? -(v) : (v))
 
-#endif // _POBJECT_H
+
+#endif // PTLIB_OBJECT_H
+
 
 // End Of File ///////////////////////////////////////////////////////////////
