@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21791 $
+ * $Revision: 23897 $
  * $Author: rjongbloed $
- * $Date: 2008-12-12 06:07:11 +0000 (Fri, 12 Dec 2008) $
+ * $Date: 2009-12-23 19:02:12 -0600 (Wed, 23 Dec 2009) $
  */
 
 #ifndef PTLIB_THREAD_H
@@ -53,9 +53,9 @@ typedef P_THREADIDENTIFIER PThreadIdentifier;
 ///////////////////////////////////////////////////////////////////////////////
 // PThread
 
-/** This class defines a thread of execution in the system. A \it thread is
+/** This class defines a thread of execution in the system. A <i>thread</i> is
    an independent flow of processor instructions. This differs from a
-   \it process which also embodies a program address space and resource
+   <i>process</i> which also embodies a program address space and resource
    allocation. So threads can share memory and resources as they run in the
    context of a given process. A process always contains at least one thread.
    This is reflected in this library by the #PProcess class being
@@ -294,6 +294,21 @@ class PThread : public PObject
     virtual PThreadIdentifier GetThreadId() const;
     static PThreadIdentifier GetCurrentThreadId();
 
+    /// Times for execution of the thread.
+    struct Times
+    {
+      PTimeInterval m_real;     ///< Total real time since thread start in milliseconds.
+      PTimeInterval m_kernel;   ///< Total kernel CPU time in milliseconds.
+      PTimeInterval m_user;     ///< Total user CPU time in milliseconds.
+      friend ostream & operator<<(ostream & strm, const Times & times);
+    };
+
+    /** Get the thread execution times.
+     */
+    bool GetTimes(
+      Times & times   ///< Times for thread execution.
+    );
+
     /** User override function for the main execution routine of the thread. A
        descendent class must provide the code that will be executed in the
        thread within this function.
@@ -423,12 +438,12 @@ class PThreadMain : public PThread
   PCLASSINFO(PThreadMain, PThread);
   public:
     typedef void (*FnType)(); 
-    PThreadMain(FnType function, bool autoDelete = false)
-      : PThread(10000, autoDelete ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread)
+    PThreadMain(FnType function, bool autoDel = false)
+      : PThread(10000, autoDel ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread)
       , m_function(function)
       { PThread::Resume(); }
-    PThreadMain(const char * file, int line, FnType function, bool autoDelete = false)
-      : PThread(10000, autoDelete ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread,  NormalPriority,
+    PThreadMain(const char * file, int line, FnType function, bool autoDel = false)
+      : PThread(10000, autoDel ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread,  NormalPriority,
                 psprintf("%s:%08x-%s:%i", GetClass(), (void *)this, file, line))
       , m_function(function)
       { PThread::Resume(); }
@@ -458,13 +473,13 @@ class PThread1Arg : public PThread
   public:
     typedef void (*FnType)(Arg1Type arg1);
 
-    PThread1Arg(Arg1Type arg1, FnType function, bool autoDelete = false)
-      : PThread(10000, autoDelete ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread)
+    PThread1Arg(Arg1Type arg1, FnType function, bool autoDel = false)
+      : PThread(10000, autoDel ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread)
       , m_function(function)
       , m_arg1(arg1)
     { PThread::Resume(); }
-    PThread1Arg(const char * file, int line, Arg1Type arg1, FnType function, bool autoDelete = false)
-      : PThread(10000, autoDelete ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread, NormalPriority,
+    PThread1Arg(const char * file, int line, Arg1Type arg1, FnType function, bool autoDel = false)
+      : PThread(10000, autoDel ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread, NormalPriority,
                 psprintf("%s:%08x-%s:%i", GetClass(), (void *)this, file, line))
       , m_function(function)
       , m_arg1(arg1)
@@ -496,14 +511,14 @@ class PThread2Arg : public PThread
   PCLASSINFO(PThread2Arg, PThread);
   public:
     typedef void (*FnType)(Arg1Type arg1, Arg2Type arg2); 
-    PThread2Arg(Arg1Type arg1, Arg2Type arg2, FnType function, bool autoDelete = false)
-      : PThread(10000, autoDelete ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread)
+    PThread2Arg(Arg1Type arg1, Arg2Type arg2, FnType function, bool autoDel = false)
+      : PThread(10000, autoDel ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread)
       , m_function(function)
       , m_arg1(arg1)
       , m_arg2(arg2)
       { PThread::Resume(); }
-    PThread2Arg(const char * file, int line, Arg1Type arg1, Arg2Type arg2, FnType function, bool autoDelete = false)
-      : PThread(10000, autoDelete ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread, NormalPriority,
+    PThread2Arg(const char * file, int line, Arg1Type arg1, Arg2Type arg2, FnType function, bool autoDel = false)
+      : PThread(10000, autoDel ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread, NormalPriority,
                 psprintf("%s:%08x-%s:%i", GetClass(), (void *)this, file, line))
       , m_function(function)
       , m_arg1(arg1)
@@ -536,15 +551,15 @@ class PThread3Arg : public PThread
   PCLASSINFO(PThread3Arg, PThread);
   public:
     typedef void (*FnType)(Arg1Type arg1, Arg2Type arg2, Arg3Type arg3); 
-    PThread3Arg(Arg1Type arg1, Arg2Type arg2, Arg3Type arg3, FnType function, bool autoDelete = false)
-      : PThread(10000, autoDelete ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread)
+    PThread3Arg(Arg1Type arg1, Arg2Type arg2, Arg3Type arg3, FnType function, bool autoDel = false)
+      : PThread(10000, autoDel ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread)
       , m_function(function)
       , m_arg1(arg1)
       , m_arg2(arg2)
       , m_arg3(arg3)
       { PThread::Resume(); }
-    PThread3Arg(const char * file, int line, Arg1Type arg1, Arg2Type arg2, Arg3Type arg3, FnType function, bool autoDelete = false)
-      : PThread(10000, autoDelete ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread, NormalPriority,
+    PThread3Arg(const char * file, int line, Arg1Type arg1, Arg2Type arg2, Arg3Type arg3, FnType function, bool autoDel = false)
+      : PThread(10000, autoDel ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread, NormalPriority,
                 psprintf("%s:%08x-%s:%i", GetClass(), (void *)this, file, line))
       , m_function(function)
       , m_arg1(arg1)
@@ -580,23 +595,30 @@ class PThread3Arg : public PThread
 template <typename ObjType>
 class PThreadObj : public PThread
 {
-  public:
-  PCLASSINFO(PThreadObj, PThread);
+    PCLASSINFO(PThreadObj, PThread);
   public:
     typedef void (ObjType::*ObjTypeFn)(); 
-    PThreadObj(ObjType & obj, ObjTypeFn function, bool autoDelete = false)
-      : PThread(10000, autoDelete ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread)
+
+    PThreadObj(
+      ObjType & obj,
+      ObjTypeFn function,
+      bool autoDel = false,
+      const char * name = NULL,
+      PThread::Priority priority = PThread::NormalPriority
+    ) : PThread(10000,
+                autoDel ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread,
+                priority,
+                name)
       , m_object(obj)
       , m_function(function)
-      { PThread::Resume(); }
-    PThreadObj(const char * file, int line, ObjType & obj, ObjTypeFn function, bool autoDelete = false)
-      : PThread(10000, autoDelete ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread, NormalPriority,
-                psprintf("%s:%08x-%s:%i", GetClass(), (void *)this, file, line))
-      , m_object(obj)
-      , m_function(function)
-      { PThread::Resume(); }
+    {
+      PThread::Resume();
+    }
+
     void Main()
-      { (m_object.*m_function)(); }
+    {
+      (m_object.*m_function)();
+    }
 
   protected:
     ObjType & m_object;
@@ -623,24 +645,32 @@ class PThreadObj : public PThread
 template <class ObjType, typename Arg1Type>
 class PThreadObj1Arg : public PThread
 {
-  PCLASSINFO(PThreadObj1Arg, PThread);
+    PCLASSINFO(PThreadObj1Arg, PThread);
   public:
     typedef void (ObjType::*ObjTypeFn)(Arg1Type); 
-    PThreadObj1Arg(ObjType & obj, Arg1Type arg1, ObjTypeFn function, bool autoDelete = false)
-      : PThread(10000, autoDelete ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread)
+
+    PThreadObj1Arg(
+      ObjType & obj,
+      Arg1Type arg1,
+      ObjTypeFn function,
+      bool autoDel = false,
+      const char * name = NULL,
+      PThread::Priority priority = PThread::NormalPriority
+    ) : PThread(10000,
+                autoDel ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread,
+                priority,
+                name)
       , m_object(obj)
       , m_function(function)
       , m_arg1(arg1)
-      { PThread::Resume(); }
-    PThreadObj1Arg(const char * file, int line, ObjType & obj, Arg1Type arg1, ObjTypeFn function, bool autoDelete = false)
-      : PThread(10000, autoDelete ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread, NormalPriority,
-                psprintf("%s:%08x-%s:%i", GetClass(), (void *)this, file, line))
-      , m_object(obj)
-      , m_function(function)
-      , m_arg1(arg1)
-      { PThread::Resume(); }
+    {
+      PThread::Resume();
+    }
+
     void Main()
-    { (m_object.*m_function)(m_arg1); }
+    {
+      (m_object.*m_function)(m_arg1);
+    }
 
   protected:
     ObjType & m_object;
@@ -651,32 +681,40 @@ class PThreadObj1Arg : public PThread
 template <class ObjType, typename Arg1Type, typename Arg2Type>
 class PThreadObj2Arg : public PThread
 {
-  PCLASSINFO(PThreadObj2Arg, PThread);
+    PCLASSINFO(PThreadObj2Arg, PThread);
   public:
-    typedef void (ObjType::*ObjTypeFn)(Arg1Type, Arg2Type); 
-    PThreadObj2Arg(ObjType & obj, Arg1Type arg1, Arg2Type arg2, ObjTypeFn function, bool autoDelete = false)
-      : PThread(10000, autoDelete ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread)
+    typedef void (ObjType::*ObjTypeFn)(Arg1Type, Arg2Type);
+
+    PThreadObj2Arg(
+      ObjType & obj,
+      Arg1Type arg1,
+      Arg2Type arg2,
+      ObjTypeFn function,
+      bool autoDel = false,
+      const char * name = NULL,
+      PThread::Priority priority = PThread::NormalPriority
+    ) : PThread(10000,
+                autoDel ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread,
+                priority,
+                name)
       , m_object(obj)
       , m_function(function)
       , m_arg1(arg1)
       , m_arg2(arg2)
-      { PThread::Resume(); }
-    PThreadObj2Arg(const char * file, int line, ObjType & obj, Arg1Type arg1, Arg2Type arg2, ObjTypeFn function, bool autoDelete = false)
-      : PThread(10000, autoDelete ? PThread::AutoDeleteThread : PThread::NoAutoDeleteThread, NormalPriority,
-                psprintf("%s:%08x-%s:%i", GetClass(), (void *)this, file, line))
-      , m_object(obj)
-      , m_function(function)
-      , m_arg1(arg1)
-      , m_arg2(arg2)
-      { PThread::Resume(); }
+    {
+      PThread::Resume();
+    }
+
     void Main()
-      { (m_object.*m_function)(m_arg1, m_arg2); }
+    {
+      (m_object.*m_function)(m_arg1, m_arg2);
+    }
 
   protected:
     ObjType & m_object;
     ObjTypeFn m_function;
     Arg1Type  m_arg1;
-    Arg1Type  m_arg2;
+    Arg2Type  m_arg2;
 };
 
 
