@@ -26,13 +26,14 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20766 $
- * $Author: rjongbloed $
- * $Date: 2008-08-29 06:41:59 +0000 (Fri, 29 Aug 2008) $
+ * $Revision: 23094 $
+ * $Author: shorne $
+ * $Date: 2009-07-10 09:08:11 -0500 (Fri, 10 Jul 2009) $
  */
 
 #include <ptlib.h>
 #include <ptclib/pnat.h>
+#include <ptclib/random.h>
 
 
 static const char PNatMethodBaseClass[] = "PNatMethod";
@@ -41,9 +42,13 @@ template <> PNatMethod * PDevicePluginFactory<PNatMethod>::Worker::Create(const
    return PNatMethod::Create(method);
 }
 
+typedef PDevicePluginAdapter<PNatMethod> PDevicePluginPNatMethod;
+PFACTORY_CREATE(PFactory<PDevicePluginAdapterBase>, PDevicePluginPNatMethod, PNatMethodBaseClass, true);
+
+
 PNatStrategy::PNatStrategy()
 {
-
+   pluginMgr = NULL;
 }
 
 PNatStrategy::~PNatStrategy()
@@ -56,16 +61,27 @@ void PNatStrategy::AddMethod(PNatMethod * method)
   natlist.Append(method);
 }
 
-PNatMethod * PNatStrategy::GetMethod()
+PNatMethod * PNatStrategy::GetMethod(const PIPSocket::Address & address)
 {
   for (PNatList::iterator i = natlist.begin(); i != natlist.end(); i++) {
-    if (i->IsAvailable())
+    if (i->IsAvailable(address))
       return &*i;
   }
 
   return NULL;
 }
 
+PNatMethod * PNatStrategy::GetMethodByName(const PString & name)
+{
+  for (PNatList::iterator i = natlist.begin(); i != natlist.end(); i++) {
+    if (i->GetName() == name) {
+      return &*i;
+	}
+  }
+
+  return NULL;
+}
+
 PBoolean PNatStrategy::RemoveMethod(const PString & meth)
 {
   for (PNatList::iterator i = natlist.begin(); i != natlist.end(); i++) {
@@ -87,32 +103,16 @@ void PNatStrategy::SetPortRanges(WORD portBase, WORD portMax, WORD portPairBase,
 
 PNatMethod * PNatStrategy::LoadNatMethod(const PString & name)
 {
-  NatFactory::KeyList_T keyList = NatFactory::GetKeyList();
-  NatFactory::KeyList_T::const_iterator r;
+   if (pluginMgr == NULL)
+    pluginMgr = &PPluginManager::GetPluginManager();
 
-   PNatMethod * dev = NULL;
-   for (r = keyList.begin(); r != keyList.end(); ++r) {
-     PString capName = *r;
-     if (name == capName) {
-       dev = NatFactory::CreateInstance(*r);
-	   AddMethod(dev);
-     }
-   }
-	
-  return dev;
+  return (PNatMethod *)pluginMgr->CreatePluginsDeviceByName(name, PNatMethodBaseClass);
 }
 
-PStringList PNatStrategy::GetRegisteredList()
+PStringArray PNatStrategy::GetRegisteredList()
 {
-  NatFactory::KeyList_T keyList = NatFactory::GetKeyList();
-  NatFactory::KeyList_T::const_iterator r;
-  PStringList methods;
-
-   for (r = keyList.begin(); r != keyList.end(); ++r) {
-        methods.AppendString(*r);
-   }
-
-   return methods;
+  PPluginManager * plugMgr = &PPluginManager::GetPluginManager();
+  return plugMgr->GetPluginsProviding(PNatMethodBaseClass);
 }
 
 ///////////////////////////////////////////////////////////////////////
@@ -135,13 +135,17 @@ PNatMethod * PNatMethod::Create(const PString & name, PPluginManager * pluginMgr
   return (PNatMethod *)pluginMgr->CreatePluginsDeviceByName(name, PNatMethodBaseClass,0);
 }
 
+PBoolean PNatMethod::CreateSocketPair(PUDPSocket * & socket1,PUDPSocket * & socket2,
+      const PIPSocket::Address & binding, void * /*userData*/)
+{
+	return CreateSocketPair(socket1,socket2,binding);
+}
 
 void PNatMethod::PrintOn(ostream & strm) const
 {
   strm << GetName() << " server " << GetServer();
 }
 
-
 PString PNatMethod::GetServer() const
 {
   PStringStream str;
@@ -189,3 +193,25 @@ void PNatMethod::SetPortRanges(WORD portBase, WORD portMax, WORD portPairBase, W
 
   pairedPortInfo.mutex.Signal();
 }
+
+void PNatMethod::Activate(bool /*active*/)
+{
+
+}
+
+void PNatMethod::SetAlternateAddresses(const PStringArray & /*addresses*/, void * /*userData*/)
+{
+
+}
+
+WORD PNatMethod::RandomPortPair(unsigned int start, unsigned int end)
+{
+	WORD num;
+	PRandom rand;
+	num = (WORD)rand.Generate(start,end);
+	if (PString(num).Right(1).FindOneOf("13579") != P_MAX_INDEX) 
+			num++;  // Make sure the number is even
+
+	return num;
+}
+
