@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24342 $
- * $Author: ededu $
- * $Date: 2010-05-14 09:27:54 -0500 (Fri, 14 May 2010) $
+ * $Revision: 24060 $
+ * $Author: rjongbloed $
+ * $Date: 2010-02-12 23:11:44 -0600 (Fri, 12 Feb 2010) $
  */
 
 #include <ptlib.h>
@@ -45,84 +45,26 @@ extern "C" int vsprintf(char *, const char *, va_list);
 #include "regex/regex.h"
 #endif
 
-#define regexpression  ((regex_t *)expression)
+#define regexpression()  ((regex_t *)expression)
 
 #if !P_USE_INLINES
 #include "ptlib/contain.inl"
 #endif
 
+
+PDEFINE_POOL_ALLOCATOR(PContainerReference);
+
+
 #define new PNEW
 #undef  __CLASS__
 #define __CLASS__ GetClass()
 
 
-
-///////////////////////////////////////////////////////////////////////////////
-//
-//  Use a custom allocator for PContainer references
-//
-
-template <class Type, class Allocator = std::allocator<Type> >
-class CustomAllocator 
-{
-  public:
-    Type * Allocate(size_t n)
-    { return (Type*)a.allocate(n); }
-
-    void Deallocate(Type * p, size_t n)
-    { a.deallocate(p, n); }
-
-    Type * Reallocate(Type * p, size_t old_n, size_t new_n)
-    { 
-      if (new_n == old_n)
-        return p;
-
-      Type * np = (Type *)a.allocate(new_n);
-      memcpy(np, p, (new_n < old_n) ? new_n : old_n );
-      a.deallocate(p, old_n); 
-
-      return np;
-    }
-
-    Allocator a;
-};
-
-#if __GNUCC__
-static CustomAllocator<PContainerReference, __gnu_cxx::pool_allocator> PContainerReference_allocator;
-#else
-static CustomAllocator<PContainerReference> PContainerReference_allocator;
-#endif
-
-static PContainerReference * PContainerReference_new(PINDEX initialSize)
-{
-  PContainerReference * ref = PContainerReference_allocator.Allocate(1);
-  if (ref != NULL) {
-    ref->size = initialSize;
-    ref->count.SetValue(1);
-    ref->deleteObjects = true;
-  }
-  return ref;
-}
-
-static PContainerReference * PContainerReference_new(const PContainerReference & r)
-{
-  PContainerReference * ref = PContainerReference_allocator.Allocate(1);
-  if (ref != NULL) {
-    ref->size = r.size;
-    ref->count.SetValue(1);
-    ref->deleteObjects = r.deleteObjects;
-  }
-  return ref;
-}
-
-static void PContainerReference_delete(PContainerReference * p)
-{ PContainerReference_allocator.Deallocate(p, 1); }
-
 ///////////////////////////////////////////////////////////////////////////////
 
 PContainer::PContainer(PINDEX initialSize)
 {
-  reference = PContainerReference_new(initialSize);
+  reference = new PContainerReference(initialSize);
   PAssert(reference != NULL, POutOfMemory);
 }
 
@@ -134,7 +76,7 @@ PContainer::PContainer(int, const PContainer * cont)
   PAssert(cont != NULL, PInvalidParameter);
   PAssert2(cont->reference != NULL, cont->GetClass(), "Clone of deleted container");
 
-  reference = PContainerReference_new(*cont->reference);
+  reference = new PContainerReference(*cont->reference);
   PAssert(reference != NULL, POutOfMemory);
 }
 
@@ -166,7 +108,7 @@ void PContainer::AssignContents(const PContainer & cont)
 
   if (--reference->count == 0) {
     DestroyContents();
-    PContainerReference_delete(reference);
+    delete reference;
     reference = NULL;
   }
 
@@ -184,7 +126,7 @@ void PContainer::Destruct()
     
     else {
       DestroyContents();
-      PContainerReference_delete(reference);
+      delete reference;
       reference = NULL;
     }
   }
@@ -208,7 +150,7 @@ PBoolean PContainer::MakeUnique()
     return PTrue;
 
   PContainerReference * oldReference = reference;
-  reference = PContainerReference_new(*reference);
+  reference = new PContainerReference(*reference);
   --oldReference->count;
 
   return PFalse;
@@ -217,13 +159,7 @@ PBoolean PContainer::MakeUnique()
 
 ///////////////////////////////////////////////////////////////////////////////
 
-#if __GNUCC__
-static CustomAllocator<char, __gnu_cxx::mt_allocator> PAbstractArray_allocator;
-#else
-static CustomAllocator<char> PAbstractArray_allocator;
-#endif
-
-
+static PVariablePoolAllocator<char> PAbstractArray_allocator;
 
 PAbstractArray::PAbstractArray(PINDEX elementSizeInBytes, PINDEX initialSize)
   : PContainer(initialSize)
@@ -234,8 +170,7 @@ PAbstractArray::PAbstractArray(PINDEX elementSizeInBytes, PINDEX initialSize)
   if (GetSize() == 0)
     theArray = NULL;
   else {
-    //theArray = (char *)calloc(GetSize(), elementSize);
-    theArray = PAbstractArray_allocator.Allocate(GetSize() * elementSize);
+    theArray = PAbstractArray_allocator.allocate(GetSize() * elementSize);
     PAssert(theArray != NULL, POutOfMemory);
     memset(theArray, 0, GetSize() * elementSize);
   }
@@ -259,7 +194,7 @@ PAbstractArray::PAbstractArray(PINDEX elementSizeInBytes,
     theArray = NULL;
   else if (dynamicAllocation) {
     PINDEX sizebytes = elementSize*GetSize();
-    theArray = PAbstractArray_allocator.Allocate(sizebytes);
+    theArray = PAbstractArray_allocator.allocate(sizebytes);
     PAssert(theArray != NULL, POutOfMemory);
     memcpy(theArray, PAssertNULL(buffer), sizebytes);
   }
@@ -271,10 +206,8 @@ PAbstractArray::PAbstractArray(PINDEX elementSizeInBytes,
 void PAbstractArray::DestroyContents()
 {
   if (theArray != NULL) {
-    if (allocatedDynamically) {
-      PAbstractArray_allocator.Deallocate(theArray, elementSize*GetSize());
-      //free(theArray);
-    }
+    if (allocatedDynamically)
+      PAbstractArray_allocator.deallocate(theArray, elementSize*GetSize());
     theArray = NULL;
   }
 }
@@ -292,7 +225,7 @@ void PAbstractArray::CloneContents(const PAbstractArray * array)
 {
   elementSize = array->elementSize;
   PINDEX sizebytes = elementSize*GetSize();
-  char * newArray = PAbstractArray_allocator.Allocate(sizebytes);
+  char * newArray = PAbstractArray_allocator.allocate(sizebytes);
   if (newArray == NULL)
     reference->size = 0;
   else
@@ -305,7 +238,7 @@ void PAbstractArray::CloneContents(const PAbstractArray * array)
 void PAbstractArray::PrintOn(ostream & strm) const
 {
   char separator = strm.fill();
-  int width = strm.width();
+  int width = (int)strm.width();
   for (PINDEX  i = 0; i < GetSize(); i++) {
     if (i > 0 && separator != '\0')
       strm << separator;
@@ -389,7 +322,7 @@ PBoolean PAbstractArray::InternalSetSize(PINDEX newSize, PBoolean force)
     if (newsizebytes == 0)
       newArray = NULL;
     else {
-      if ((newArray = PAbstractArray_allocator.Allocate(newsizebytes)) == NULL)
+      if ((newArray = PAbstractArray_allocator.allocate(newsizebytes)) == NULL)
         return PFalse;
   
       if (theArray != NULL)
@@ -397,29 +330,27 @@ PBoolean PAbstractArray::InternalSetSize(PINDEX newSize, PBoolean force)
     }
 
     --reference->count;
-    reference = PContainerReference_new(newSize);
+    reference = new PContainerReference(newSize);
 
   } else {
 
     if (theArray != NULL) {
       if (newsizebytes == 0) {
         if (allocatedDynamically)
-          PAbstractArray_allocator.Deallocate(theArray, oldsizebytes);
+          PAbstractArray_allocator.deallocate(theArray, oldsizebytes);
         newArray = NULL;
       }
-      else if (allocatedDynamically) {
-        if ((newArray = (char *)PAbstractArray_allocator.Reallocate(theArray, oldsizebytes, newsizebytes)) == NULL)
-          return PFalse;
-      }
       else {
-        if ((newArray = PAbstractArray_allocator.Allocate(newsizebytes)) == NULL)
-          return PFalse;
+        if ((newArray = PAbstractArray_allocator.allocate(newsizebytes)) == NULL)
+          return false;
         memcpy(newArray, theArray, PMIN(newsizebytes, oldsizebytes));
-        allocatedDynamically = PTrue;
+        if (allocatedDynamically)
+          PAbstractArray_allocator.deallocate(theArray, oldsizebytes);
+        allocatedDynamically = true;
       }
     }
     else if (newsizebytes != 0) {
-      if ((newArray = PAbstractArray_allocator.Allocate(newsizebytes)) == NULL)
+      if ((newArray = PAbstractArray_allocator.allocate(newsizebytes)) == NULL)
         return PFalse;
     }
     else
@@ -438,7 +369,7 @@ PBoolean PAbstractArray::InternalSetSize(PINDEX newSize, PBoolean force)
 void PAbstractArray::Attach(const void *buffer, PINDEX bufferSize)
 {
   if (allocatedDynamically && theArray != NULL)
-    PAbstractArray_allocator.Deallocate(theArray, elementSize*GetSize());
+    PAbstractArray_allocator.deallocate(theArray, elementSize*GetSize());
 
   theArray = (char *)buffer;
   reference->size = bufferSize;
@@ -483,7 +414,7 @@ void PAbstractArray::ReadElementFrom(istream & /*stream*/, PINDEX /*index*/)
 
 void PCharArray::PrintOn(ostream & strm) const
 {
-  PINDEX width = strm.width();
+  PINDEX width = (int)strm.width();
   if (width > GetSize())
     width -= GetSize();
   else
@@ -518,12 +449,12 @@ void PCharArray::ReadFrom(istream &strm)
 
 void PBYTEArray::PrintOn(ostream & strm) const
 {
-  PINDEX line_width = strm.width();
+  PINDEX line_width = (int)strm.width();
   if (line_width == 0)
     line_width = 16;
   strm.width(0);
 
-  PINDEX indent = strm.precision();
+  PINDEX indent = (int)strm.precision();
 
   PINDEX val_width = ((strm.flags()&ios::basefield) == ios::hex) ? 2 : 3;
 
@@ -1508,11 +1439,17 @@ PBoolean PString::FindRegEx(const PRegularExpression & regex,
                         PINDEX offset,
                         PINDEX maxPos) const
 {
-  if (offset < 0 || maxPos < 0 || offset >= GetLength())
-    return PFalse;
+  if (offset < 0 || maxPos < 0 || offset > GetLength())
+    return false;
 
-  if (!regex.Execute(&theArray[offset], pos, len, 0))
-    return PFalse;
+  if (offset == GetLength()) {
+    if (!regex.Execute("", pos, len, 0))
+      return false;
+  }
+  else {
+    if (!regex.Execute(&theArray[offset], pos, len, 0))
+      return PFalse;
+  }
 
   pos += offset;
   if (pos+len > maxPos)
@@ -1521,6 +1458,16 @@ PBoolean PString::FindRegEx(const PRegularExpression & regex,
   return PTrue;
 }
 
+PBoolean PString::MatchesRegEx(const PRegularExpression & regex) const
+{
+  PINDEX pos = 0;
+  PINDEX len = 0;
+
+  if (!regex.Execute(theArray, pos, len, 0))
+    return PFalse;
+
+  return (pos == 0) && (len == GetLength());
+}
 
 void PString::Replace(const PString & target,
                       const PString & subs,
@@ -1553,6 +1500,8 @@ void PString::Splice(const char * cstr, PINDEX pos, PINDEX len)
     operator+=(cstr);
   else {
     MakeUnique();
+    if (len > slen-pos)
+      len = slen-pos;
     PINDEX clen = cstr != NULL ? strlen(cstr) : 0;
     PINDEX newlen = slen-len+clen;
     if (clen > len)
@@ -1752,7 +1701,7 @@ PWCharArray PString::AsUCS2() const
   gsize g_len = 0;
   gchar * g_ucs2 = g_convert(theArray, GetSize()-1, "UCS-2", "UTF-8", 0, &g_len, 0);
   if (g_ucs2 != NULL) {
-    if (ucs2.SetSize(g_len))
+    if (ucs2.SetSize(g_len+1))
       memcpy(ucs2.GetPointer(), g_ucs2, g_len*2);
     g_free(g_ucs2);
     return ucs2;
@@ -2110,7 +2059,7 @@ streambuf::pos_type PStringStream::Buffer::seekoff(off_type off, ios_base::seekd
   if ((mode&ios::out) != 0)
     setp(newpptr, epptr());
 
-  return 0;
+  return gptr() - eback();
 }
 
 
@@ -2165,7 +2114,7 @@ PStringStream::~PStringStream()
 
 PString & PStringStream::MakeEmpty()
 {
-  *theArray = '\0';
+  memset(theArray, 0, GetSize());
   flush();
   return *this;
 }
@@ -2657,8 +2606,8 @@ PRegularExpression & PRegularExpression::operator =(const PRegularExpression & f
 {
   if (expression != from.expression) {
     if (expression != NULL && expression != from.expression) {
-      regfree(regexpression);
-      delete regexpression;
+      regfree(regexpression());
+      delete regexpression();
     }
     expression = NULL;
   }
@@ -2669,8 +2618,8 @@ PRegularExpression & PRegularExpression::operator =(const PRegularExpression & f
 PRegularExpression::~PRegularExpression()
 {
   if (expression != NULL) {
-    regfree(regexpression);
-    delete regexpression;
+    regfree(regexpression());
+    delete regexpression();
   }
 }
 
@@ -2684,7 +2633,7 @@ PRegularExpression::ErrorCodes PRegularExpression::GetErrorCode() const
 PString PRegularExpression::GetErrorText() const
 {
   PString str;
-  regerror(lastError, regexpression, str.GetPointer(256), 256);
+  regerror(lastError, regexpression(), str.GetPointer(256), 256);
   return str;
 }
 
@@ -2701,15 +2650,15 @@ PBoolean PRegularExpression::Compile(const char * pattern, int flags)
   flagsSaved   = flags;
 
   if (expression != NULL) {
-    regfree(regexpression);
-    delete regexpression;
+    regfree(regexpression());
+    delete regexpression();
     expression = NULL;
   }
   if (pattern == NULL || *pattern == '\0')
     lastError = BadPattern;
   else {
     expression = new regex_t;
-    lastError = (ErrorCodes)regcomp(regexpression, pattern, flags);
+    lastError = (ErrorCodes)regcomp(regexpression(), pattern, flags);
   }
   return lastError == NoError;
 }
@@ -2747,7 +2696,7 @@ PBoolean PRegularExpression::Execute(const char * cstr, PINDEX & start, PINDEX &
 
   regmatch_t match;
 
-  lastError = (ErrorCodes)regexec(regexpression, cstr, 1, &match, flags);
+  lastError = (ErrorCodes)regexec(regexpression(), cstr, 1, &match, flags);
   if (lastError != NoError)
     return PFalse;
 
@@ -2799,7 +2748,7 @@ PBoolean PRegularExpression::Execute(const char * cstr,
   else
     count = 1;
 
-  lastError = (ErrorCodes)regexec(regexpression, cstr, count, matches, flags);
+  lastError = (ErrorCodes)regexec(regexpression(), cstr, count, matches, flags);
   if (lastError == NoError) {
     starts.SetMinSize(count);
     ends.SetMinSize(count);
@@ -2818,19 +2767,16 @@ PBoolean PRegularExpression::Execute(const char * cstr,
 
 PString PRegularExpression::EscapeString(const PString & str)
 {
-  PString translated;
+  PString translated = str;
 
   PINDEX lastPos = 0;
   PINDEX nextPos;
-  while ((nextPos = str.FindOneOf("\\^$+?*.[]()|{}", lastPos+1)) != P_MAX_INDEX) {
-    translated += str(lastPos, nextPos-1) + "\\";
-    lastPos = nextPos;
+  while ((nextPos = translated.FindOneOf("\\^$+?*.[]()|{}", lastPos)) != P_MAX_INDEX) {
+    translated.Splice("\\", nextPos);
+    lastPos = nextPos+2;
   }
 
-  if (lastPos == 0)
-    return str;
-
-  return translated + str.Mid(lastPos);
+  return translated;
 }
 
 
