@@ -26,13 +26,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24331 $
- * $Author: ededu $
- * $Date: 2010-05-07 13:36:59 -0500 (Fri, 07 May 2010) $
+ * $Revision: 24265 $
+ * $Author: rjongbloed $
+ * $Date: 2010-04-22 23:49:18 -0500 (Thu, 22 Apr 2010) $
  */
 
 #include <ptlib.h>
-
+#include <ptbuildopts.h>
 #include <ptlib/sockets.h>
 
 #include <ctype.h>
@@ -92,7 +92,11 @@ static int defaultIPv6ScopeId = 0;
 
 static PIPSocket::Address loopback6(16,(const BYTE *)"\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\001");
 static PIPSocket::Address broadcast6(16,(const BYTE *)"\377\002\0\0\0\0\0\0\0\0\0\0\0\0\0\001"); // IPV6 multicast address
-static PIPSocket::Address any6(16,(const BYTE *)"\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0"); 
+static PIPSocket::Address any6(16,(const BYTE *)"\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0");
+
+#define IPV6_PARAM(p) p
+#else
+#define IPV6_PARAM(p)
 #endif
 
 
@@ -164,32 +168,37 @@ PIPSocket::Address PIPSocket::GetDefaultIpAny()
 class Psockaddr
 {
   public:
-    Psockaddr() { memset(&storage, 0, sizeof(storage)); }
+    Psockaddr() : ptr(&storage) { memset(&storage, 0, sizeof(storage)); }
     Psockaddr(const PIPSocket::Address & ip, WORD port);
-    sockaddr* operator->() const { return (sockaddr *)&storage; }
-    operator sockaddr*()   const { return (sockaddr *)&storage; }
+    sockaddr* operator->() const { return addr; }
+    operator sockaddr*()   const { return addr; }
     socklen_t GetSize() const;
     PIPSocket::Address GetIP() const;
     WORD GetPort() const;
   private:
     sockaddr_storage storage;
+    union {
+      sockaddr_storage * ptr;
+      sockaddr         * addr;
+      sockaddr_in      * addr4;
+      sockaddr_in6     * addr6;
+    };
 };
 
 
 Psockaddr::Psockaddr(const PIPSocket::Address & ip, WORD port)
+ : ptr(&storage) 
 {
   memset(&storage, 0, sizeof(storage));
 
   if (ip.GetVersion() == 6) {
-    sockaddr_in6 * addr6 = (sockaddr_in6 *)&storage;
     addr6->sin6_family = AF_INET6;
     addr6->sin6_addr = ip;
     addr6->sin6_port = htons(port);
     addr6->sin6_flowinfo = 0;
-	addr6->sin6_scope_id = PIPSocket::GetDefaultV6ScopeId(); // Should be set to the right interface....
+    addr6->sin6_scope_id = PIPSocket::GetDefaultV6ScopeId(); // Should be set to the right interface....
   }
   else {
-    sockaddr_in * addr4 = (sockaddr_in *)&storage;
     addr4->sin_family = AF_INET;
     addr4->sin_addr = ip;
     addr4->sin_port = htons(port);
@@ -199,7 +208,7 @@ Psockaddr::Psockaddr(const PIPSocket::Address & ip, WORD port)
 
 socklen_t Psockaddr::GetSize() const
 {
-  switch (((sockaddr *)&storage)->sa_family) {
+  switch (addr->sa_family) {
     case AF_INET :
       return sizeof(sockaddr_in);
     case AF_INET6 :
@@ -214,11 +223,11 @@ socklen_t Psockaddr::GetSize() const
 
 PIPSocket::Address Psockaddr::GetIP() const
 {
-  switch (((sockaddr *)&storage)->sa_family) {
+  switch (addr->sa_family) {
     case AF_INET :
-      return ((sockaddr_in *)&storage)->sin_addr;
+      return addr4->sin_addr;
     case AF_INET6 :
-      return ((sockaddr_in6 *)&storage)->sin6_addr;
+      return addr6->sin6_addr;
     default :
       return 0;
   }
@@ -227,11 +236,11 @@ PIPSocket::Address Psockaddr::GetIP() const
 
 WORD Psockaddr::GetPort() const
 {
-  switch (((sockaddr *)&storage)->sa_family) {
+  switch (addr->sa_family) {
     case AF_INET :
-      return ntohs(((sockaddr_in *)&storage)->sin_port);
+      return ntohs(addr4->sin_port);
     case AF_INET6 :
-      return ntohs(((sockaddr_in6 *)&storage)->sin6_port);
+      return ntohs(addr6->sin6_port);
     default :
       return 0;
   }
@@ -1522,20 +1531,13 @@ PBoolean PIPSocket::Listen(const Address & bindAddr,
 #ifndef P_BEOS
 #if P_HAS_IPV6
   if(bind_sa->sa_family != AF_INET6)
-  {
-	 // attempt to listen
-	 if (!SetOption(SO_REUSEADDR, reuse == CanReuseAddress ? 1 : 0)) {
-	   os_close();
-	   return PFalse;
-	 }
-  }
 #endif
-#else
-  // attempt to listen
-  int value = reuse == CanReuseAddress ? 1 : 0;
-  if (!SetOption(SO_REUSEADDR, &value, sizeof(int))) {
-    os_close();
-    return PFalse;
+  {
+   // attempt to listen
+   if (!SetOption(SO_REUSEADDR, reuse == CanReuseAddress ? 1 : 0)) {
+     os_close();
+     return PFalse;
+   }
   }
 #endif // BEOS
 
@@ -1594,7 +1596,7 @@ PBoolean PIPSocket::Listen(const Address & bindAddr,
     if (ConvertOSError(::getsockname(os_handle, (struct sockaddr*)&sin, &size))) {
       port = ntohs(sin.sin_port);
 
-	  if (!IN_MULTICAST(ntohl(sin.sin_addr.s_addr)))
+    if (!IN_MULTICAST(ntohl(sin.sin_addr.s_addr)))
         return true;
 
       struct ip_mreq  mreq;
@@ -1625,41 +1627,35 @@ PBoolean PIPSocket::Address::IsV4Mapped() const
   return IN6_IS_ADDR_V4MAPPED(&v.six) || IN6_IS_ADDR_V4COMPAT(&v.six);
 }
 
+#endif
 
-const PIPSocket::Address & PIPSocket::Address::GetLoopback(int version)
-{
-  return version == 6 ? loopback6 : loopback4;
-}
-
-const PIPSocket::Address & PIPSocket::Address::GetAny(int version)
-{
-  return version == 6 ? any6 : any4;
-}
-
-const PIPSocket::Address PIPSocket::Address::GetBroadcast(int version)
-{
-  return version == 6 ? broadcast6 : broadcast4;
-}
-
-#else
-
-const PIPSocket::Address & PIPSocket::Address::GetLoopback(int)
+const PIPSocket::Address & PIPSocket::Address::GetLoopback(int IPV6_PARAM(version))
 {
+#if P_HAS_IPV6
+  if (version == 6)
+    return loopback6;
+#endif
   return loopback4;
 }
 
-const PIPSocket::Address & PIPSocket::Address::GetAny(int)
+const PIPSocket::Address & PIPSocket::Address::GetAny(int IPV6_PARAM(version))
 {
+#if P_HAS_IPV6
+  if (version == 6)
+    return any6;
+#endif
   return any4;
 }
 
-const PIPSocket::Address PIPSocket::Address::GetBroadcast(int)
+const PIPSocket::Address PIPSocket::Address::GetBroadcast(int IPV6_PARAM(version))
 {
+#if P_HAS_IPV6
+  if (version == 6)
+    return broadcast6;
+#endif
   return broadcast4;
 }
 
-#endif
-
 
 PBoolean PIPSocket::Address::IsAny() const
 {
@@ -1670,11 +1666,11 @@ PBoolean PIPSocket::Address::IsAny() const
 PIPSocket::Address::Address()
 {
 #if P_HAS_IPV6
-	if(defaultIpAddressFamily == AF_INET6)
-		*this = loopback6;
-	else
+  if(defaultIpAddressFamily == AF_INET6)
+    *this = loopback6;
+  else
 #endif
-		*this = loopback4;
+    *this = loopback4;
 }
 
 
@@ -1864,7 +1860,7 @@ PIPSocket::Address & PIPSocket::Address::operator=(const PString & dotNotation)
 }
 
 
-PString PIPSocket::Address::AsString(bool bracketIPv6) const
+PString PIPSocket::Address::AsString(bool IPV6_PARAM(bracketIPv6)) const
 {
 #if defined(P_VXWORKS)
   char ipStorage[INET_ADDR_LEN];
@@ -1882,7 +1878,7 @@ PString PIPSocket::Address::AsString(bool bracketIPv6) const
     str.MakeMinimumSize();
     if (bracketIPv6)
       return '[' + str + ']';
-	return str;
+    return str;
   }
 #endif // P_HAS_IPV6
 # if defined(P_HAS_INET_NTOP)
@@ -2609,11 +2605,8 @@ PQoS & PUDPSocket::GetQoSSpec()
 
 PBoolean PUDPSocket::ApplyQoS()
 {
-#ifdef _WIN32_WCE
-  return PFalse;   //QoS not supported
-#endif
-
   char DSCPval = 0;
+#ifndef _WIN32_WCE
   if (qosSpec.GetDSCP() < 0 ||
       qosSpec.GetDSCP() > 63) {
     if (qosSpec.GetServiceType() == SERVICETYPE_PNOTDEFINED)
@@ -2635,12 +2628,17 @@ PBoolean PUDPSocket::ApplyQoS()
   }
   else
     DSCPval = (char)qosSpec.GetDSCP();
+#else
+  DSCPval = 0x38;
+  disableGQoS = PFalse;
+#endif
 
 #ifdef _WIN32
 #if P_QOS
   if (disableGQoS)
     return PFalse;
 
+#ifndef _WIN32_WCE
   PBoolean usesetsockopt = PFalse;
 
   OSVERSIONINFO versInfo;
@@ -2661,6 +2659,9 @@ PBoolean PUDPSocket::ApplyQoS()
         versInfo.dwMinorVersion == 0)
       usesetsockopt = PTrue;         //Windows 2000 does not always support QOS_DESTADDR
   }
+#else
+  PBoolean usesetsockopt = PTrue;
+#endif
 
   PBoolean retval = PFalse;
   if (!usesetsockopt && sendAddress.IsValid() && sendPort != 0) {
@@ -2717,10 +2718,6 @@ PBoolean PUDPSocket::ApplyQoS()
 
 PBoolean PUDPSocket::OpenSocketGQOS(int af, int type, int proto)
 {
-#ifdef _WIN32_WCE   //QOS not supported
-  return ConvertOSError(os_handle = os_socket(af, type, proto));
-#endif
-
 #if defined(_WIN32) && defined(P_QOS)
     
   DWORD bufferSize = 0;
@@ -2787,7 +2784,6 @@ PBoolean PUDPSocket::OpenSocketGQOS(int af, int type, int proto)
 }
 
 #ifdef _WIN32
-#ifndef _WIN32_WCE
 #if P_QOS
 
 #define COULD_HAVE_QOS
@@ -2806,7 +2802,6 @@ static PBoolean CheckOSVersionFor(DWORD major, DWORD minor)
 }
 
 #endif // P_QOS
-#endif // _WIN32_WCE
 #endif // _WIN32
 
 PBoolean PUDPSocket::OpenSocket()
@@ -2878,6 +2873,11 @@ void PUDPSocket::GetLastReceiveAddress(Address & address, WORD & port)
   port    = lastReceivePort;
 }
 
+PBoolean PUDPSocket::IsAlternateAddress(const Address &, WORD)
+{
+  return false;
+}
+
 //////////////////////////////////////////////////////////////////////////////
 
 PBoolean PICMPSocket::OpenSocket(int)
