@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22240 $
+ * $Revision: 23936 $
  * $Author: rjongbloed $
- * $Date: 2009-03-22 19:54:33 -0500 (Sun, 22 Mar 2009) $
+ * $Date: 2010-01-13 20:36:07 -0600 (Wed, 13 Jan 2010) $
  */
 
 #include <ptlib.h>
@@ -1541,8 +1541,8 @@ PString PIPSocket::GetGatewayInterface(int version)
 
 PIPSocket::Address PIPSocket::GetGatewayInterfaceAddress(int version)
 {
-#if IPv6_ENABLED
   if (version == 6) {
+#if IPv6_ENABLED
     PIPRouteTableIPv6 routes;
     if (routes->NumEntries > 0) {
       PIPAdaptersAddressTable interfaces;
@@ -1565,8 +1565,10 @@ PIPSocket::Address PIPSocket::GetGatewayInterfaceAddress(int version)
         }
       }
     }
-  }
+#else
+    return GetDefaultIpAny();
 #endif
+  }
 
   PIPRouteTable routes;
   for (unsigned i = 0; i < routes->dwNumEntries; ++i) {
@@ -1614,57 +1616,77 @@ PBoolean PIPSocket::GetRouteTable(RouteTable & table)
 #pragma optimize("g", off)
 #endif
 
-bool PIPSocket::WaitForRouteTableChange(const PTimeInterval & timeout, PSyncPoint * cancellation)
+class Win32RouteTableDetector : public PIPSocket::RouteTableDetector
 {
-  HANDLE handle = NULL;
-  OVERLAPPED overlap;
-  memset(&overlap, 0, sizeof(overlap));
+    PDynaLink  m_dll;
+    BOOL    (* m_pCancelIPChangeNotify )(LPOVERLAPPED);
+    HANDLE     m_hCancel;
 
-  // Do this so compatible with older operating systems
-  PDynaLink dll("iphlpapi.dll");
-  BOOL (*pCancelIPChangeNotify)(LPOVERLAPPED) = NULL;
-  bool doNotify = dll.GetFunction("CancelIPChangeNotify", (PDynaLink::Function&)pCancelIPChangeNotify);
-
-  if (doNotify) {
-    DWORD error = NotifyAddrChange(&handle, &overlap);
-    if (error != ERROR_IO_PENDING) {
-      PTRACE(1, "PTlib\tCould not get network interface change notification: error=" << error);
-      doNotify = false;
+  public:
+    Win32RouteTableDetector()
+      : m_dll("iphlpapi.dll")
+      , m_hCancel(CreateEvent(NULL, TRUE, FALSE, NULL))
+    {
+      if (!m_dll.GetFunction("CancelIPChangeNotify", (PDynaLink::Function&)m_pCancelIPChangeNotify))
+        m_pCancelIPChangeNotify = NULL;
     }
-  }
 
-  if (!doNotify) {
-    if (cancellation != NULL)
-      return cancellation->Wait(timeout);
+    ~Win32RouteTableDetector()
+    {
+      if (m_hCancel != NULL)
+        CloseHandle(m_hCancel);
+    }
 
-    PThread::Sleep(timeout);
-    return false;
-  }
+    virtual bool Wait(const PTimeInterval & timeout)
+    {
+      HANDLE hNotify = NULL;
+      OVERLAPPED overlap;
 
-  if (cancellation == NULL)
-    return WaitForSingleObject(handle, timeout.GetInterval()) == WAIT_OBJECT_0;
+      if (m_pCancelIPChangeNotify != NULL) {
+        memset(&overlap, 0, sizeof(overlap));
+        DWORD error = NotifyAddrChange(&hNotify, &overlap);
+        if (error != ERROR_IO_PENDING) {
+          PTRACE(1, "PTlib\tCould not get network interface change notification: error=" << error);
+          hNotify = NULL;
+        }
+      }
 
-  HANDLE handles[2];
-  handles[0] = handle;
-  handles[1] = cancellation->GetHandle();
-  switch (WaitForMultipleObjects(2, handles, false, timeout.GetInterval())) {
-    case WAIT_OBJECT_0 :
-      return true;
+      if (hNotify == NULL)
+        return WaitForSingleObject(m_hCancel, timeout.GetInterval()) == WAIT_TIMEOUT;
 
-    case WAIT_OBJECT_0+1 :
-      pCancelIPChangeNotify(&overlap);
-      // Do next case
+      HANDLE handles[2];
+      handles[0] = hNotify;
+      handles[1] = m_hCancel;
+      switch (WaitForMultipleObjects(2, handles, false, INFINITE)) {
+        case WAIT_OBJECT_0 :
+          return true;
 
-    default :
-      return false;
-  }
-}
+        case WAIT_OBJECT_0+1 :
+          m_pCancelIPChangeNotify(&overlap);
+          // Do next case
+
+        default :
+          return false;
+      }
+    }
+
+    virtual void Cancel()
+    {
+      SetEvent(m_hCancel);
+    }
+};
 
 #ifdef _MSC_VER
 #pragma optimize("", on)
 #endif
 
 
+PIPSocket::RouteTableDetector * PIPSocket::CreateRouteTableDetector()
+{
+  return new Win32RouteTableDetector();
+}
+
+
 PIPSocket::Address PIPSocket::GetRouteAddress(PIPSocket::Address remoteAddress)
 {
   DWORD best;
