@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22295 $
+ * $Revision: 23931 $
  * $Author: rjongbloed $
- * $Date: 2009-03-25 19:31:31 -0500 (Wed, 25 Mar 2009) $
+ * $Date: 2010-01-13 06:19:08 -0600 (Wed, 13 Jan 2010) $
  */
 
 #pragma implementation "sockets.h"
@@ -40,7 +40,6 @@
 #pragma implementation "ethsock.h"
 #pragma implementation "qos.h"
 
-
 #include <ptlib.h>
 #include <ptlib/sockets.h>
 
@@ -219,7 +218,7 @@ PBoolean PSocket::os_accept(PSocket & listener, struct sockaddr * addr, PINDEX *
     if (errno != EPROTO)
       return ConvertOSError(-1);
 
-    PTRACE(3, "PWLib\tAccept on " << sock << " failed with EPROTO - retrying");
+    PTRACE(3, "PTLib\tAccept on " << sock << " failed with EPROTO - retrying");
   }
 #else
   return ConvertOSError(os_handle = SetNonBlocking(::accept(listener.GetHandle(), addr, (socklen_t *)size)));
@@ -337,8 +336,8 @@ PChannel::Errors PSocket::Select(SelectList & read,
       if (fds[0].IsPresent(unblockPipe)) {
         PTRACE(6, "PWLib\tSelect unblocked fd=" << unblockPipe);
         BYTE ch;
-        ::read(unblockPipe, &ch, 1);
-        lastError = Interrupted;
+        if (ConvertOSError(::read(unblockPipe, &ch, 1), lastError, osError))
+          lastError = Interrupted;
       }
     }
   }
@@ -593,7 +592,7 @@ PBoolean PSocket::os_recvfrom(
   // read a packet 
   int r = ::recvmsg(os_handle, &readData, flags);
   if (r == -1) {
-    PTRACE(5, "PWLIB\trecvmsg returned error " << r);
+    PTRACE(5, "PTLIB\trecvmsg returned error " << r);
     ::recvmsg(os_handle, &readData, MSG_ERRQUEUE);
   }
 
@@ -1539,16 +1538,147 @@ PBoolean PIPSocket::GetRouteTable(RouteTable & table)
 #endif
 
 
-bool PIPSocket::WaitForRouteTableChange(const PTimeInterval & timeout, PSyncPoint * cancellation)
-{
-  if (cancellation != NULL)
-    return cancellation->Wait(timeout);
 
-  // Need an implementation here!!!!!!!
-  PThread::Sleep(timeout);
-  return false;
+#ifdef P_HAS_NETLINK
+
+#include <asm/types.h>
+#include <sys/socket.h>
+#include <linux/netlink.h>
+#include <linux/rtnetlink.h>
+#include <linux/genetlink.h>
+
+#include <memory.h>
+#include <errno.h>
+
+class NetLinkRouteTableDetector : public PIPSocket::RouteTableDetector
+{
+  public:
+    int m_fdLink;
+    int m_fdCancel[2];
+
+    NetLinkRouteTableDetector()
+    {
+      m_fdLink = socket(AF_NETLINK, SOCK_RAW, NETLINK_ROUTE);
+
+      if (m_fdLink != -1) {
+        struct sockaddr_nl sanl;
+        memset(&sanl, 0, sizeof(sanl));
+        sanl.nl_family = AF_NETLINK;
+        sanl.nl_groups = RTMGRP_LINK | RTMGRP_IPV4_IFADDR;
+
+        bind(m_fdLink, (struct sockaddr *)&sanl, sizeof(sanl));
+      }
+
+      if (pipe(m_fdCancel) == -1)
+        m_fdCancel[0] = m_fdCancel[1] = -1;
+
+      PTRACE(3, "PTLIB\tOpened NetLink socket");
+    }
+
+    ~NetLinkRouteTableDetector()
+    {
+      if (m_fdLink != -1)
+        close(m_fdLink);
+      if (m_fdCancel[0] != -1)
+        close(m_fdCancel[0]);
+      if (m_fdCancel[1] != -1)
+        close(m_fdCancel[1]);
+    }
+
+    bool Wait(const PTimeInterval & timeout)
+    {
+      if (m_fdCancel[0] == -1)
+        return false;
+
+      bool ok = true;
+      while (ok) {
+        fd_set fds;
+        FD_ZERO(&fds);
+        FD_SET(m_fdCancel[0], &fds);
+
+        struct timeval tval;
+        struct timeval * ptval = NULL;
+        if (m_fdLink != -1) {
+          tval.tv_sec  = timeout.GetMilliSeconds() / 1000;
+          tval.tv_usec = (timeout.GetMilliSeconds() % 1000) * 1000;
+          ptval = &tval;
+
+          FD_SET(m_fdLink, &fds);
+        }
+
+        int result = select(std::max(m_fdLink, m_fdCancel[0])+1, &fds, NULL, NULL, ptval);
+        if (result < 0)
+          return false;
+        if (result == 0)
+          return true;
+
+        if (FD_ISSET(m_fdCancel[0], &fds))
+          return false;
+
+        struct sockaddr_nl snl;
+        char buf[4096];
+        struct iovec iov = { buf, sizeof buf };
+        struct msghdr msg = { (void*)&snl, sizeof snl, &iov, 1, NULL, 0, 0};
+
+        int status = recvmsg(m_fdLink, &msg, 0);
+        if (status < 0)
+          return false;
+
+        for (struct nlmsghdr * nlmsg = (struct nlmsghdr *)buf;
+             NLMSG_OK(nlmsg, (unsigned)status);
+             nlmsg = NLMSG_NEXT(nlmsg, status)) {
+          if (nlmsg->nlmsg_len < sizeof(struct nlmsghdr))
+            break;
+
+          switch (nlmsg->nlmsg_type) {
+            case RTM_NEWADDR :
+            case RTM_DELADDR :
+              PTRACE(3, "PTLIB\tInterface table change detected via NetLink");
+              return true;
+          }
+        }
+      }
+      return false;
+    }
+
+    void Cancel()
+    {
+      PAssert(write(m_fdCancel[1], "", 1) == 1, POperatingSystemError);
+    }
+};
+
+PIPSocket::RouteTableDetector * PIPSocket::CreateRouteTableDetector()
+{
+  return new NetLinkRouteTableDetector();
 }
 
+#else
+
+class DummyRouteTableDetector : public PIPSocket::RouteTableDetector
+{
+  public:
+    bool Wait(const PTimeInterval & timeout)
+    {
+      return !m_cancel.Wait(timeout);
+    }
+
+    void Cancel()
+    {
+      m_cancel.Signal();
+    }
+
+  private:
+    PSyncPoint m_cancel;
+};
+
+
+PIPSocket::RouteTableDetector * PIPSocket::CreateRouteTableDetector()
+{
+  return new DummyRouteTableDetector();
+}
+
+#endif
+
 
 // fe800000000000000202e3fffe1ee330 02 40 20 80     eth0
 // 00000000000000000000000000000001 01 80 10 80       lo
