@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23336 $
+ * $Revision: 23344 $
  * $Author: rjongbloed $
- * $Date: 2009-09-04 20:02:14 -0500 (Fri, 04 Sep 2009) $
+ * $Date: 2009-09-05 01:12:00 -0500 (Sat, 05 Sep 2009) $
  */
 
 #define _OSUTIL_CXX
@@ -101,8 +101,6 @@ extern "C" int select(int width,
 #include "OS.h"
 #endif
 
-#include "../common/pglobalstatic.cxx"
-
 
 PString PProcess::GetOSClass()
 {
@@ -399,9 +397,9 @@ extern PProcess * PProcessInstance;;
 PString PX_GetThreadName(pthread_t id)
 {
   if (PProcessInstance != NULL) {
-    PWaitAndSignal m(PProcessInstance->activeThreadMutex);
-    PThread & thread = PProcessInstance->activeThreads[_hptr(id)];
-    return thread.GetThreadName();
+    PWaitAndSignal m(PProcessInstance->m_activeThreadMutex);
+    PThread * thread = PProcessInstance->m_activeThreads[id];
+    return thread->GetThreadName();
   }
   return psprintf("%08x", id);
 }
@@ -528,22 +526,18 @@ void PProcess::PXOnSignal(int sig)
     PMemoryHeap::GetState(state);
 #endif
     PStringStream strm;
-    activeThreadMutex.Wait();
-    PINDEX i;
-    strm << "===============\n";
-    strm << activeThreads.GetSize() << " active threads\n";
-    for (i = 0; i < activeThreads.GetSize(); ++i) {
-      POrdinalKey key = activeThreads.GetKeyAt(i);
-      PThread & thread = activeThreads[key];
-      strm << "  " << thread << "\n";
-    }
+    m_activeThreadMutex.Wait();
+    strm << "===============\n"
+         << m_activeThreads.size() << " active threads\n";
+    for (ThreadMap::iterator it = m_activeThreads.begin(); it != m_activeThreads.end(); ++it)
+      strm << "  " << *it->second << "\n";
 #if PMEMORY_CHECK
     strm << "---------------\n";
     PMemoryHeap::DumpObjectsSince(state, strm);
     PMemoryHeap::GetState(state);
 #endif
     strm << "===============\n";
-    activeThreadMutex.Signal();
+    m_activeThreadMutex.Signal();
     fprintf(stderr, "%s", strm.GetPointer());
 #if PMEMORY_CHECK
     PMemoryHeap::SetIgnoreAllocations(oldIgnore);
